/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import jama.Matrix;

import java.io.IOException;
import java.util.ArrayList;

import jgromacs.analysis.Distances;
import jgromacs.analysis.GNM;
import jgromacs.data.Structure;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class GNMTest extends TestCase {

	
	@Test
	public void testGetKirchhoffMatrix() {
		
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			Structure[] chains = s.getChains();
			Structure input = chains[0];
			input = input.getSubStructure(input.getProteinIndexSet());
			
			GNM gnm = new GNM(input,0.7,Distances.ALPHACARBON);
			
			Matrix M = gnm.getKirchhoffMatrix();

			
			// Correct values are calculated with the oGNM webserver
			assertEquals(3, M.get(0, 0),0.00001);
			assertEquals(5, M.get(1, 1),0.00001);
			assertEquals(6, M.get(2, 2),0.00001);
			assertEquals(10, M.get(43, 43),0.00001);
			assertEquals(2, M.get(102, 102),0.00001);
			assertEquals(-1, M.get(14, 94),0.00001);
			assertEquals(-1, M.get(5, 6),0.00001);
			assertEquals(-1, M.get(13, 96),0.00001);
			assertEquals(0, M.get(11, 14),0.00001);
			assertEquals(0, M.get(26, 49),0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetLambdaMatrix() {

		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			Structure[] chains = s.getChains();
			Structure input = chains[0];
			input = input.getSubStructure(input.getAlphaCarbonIndexSet());	
				
			GNM gnm = new GNM(input,0.7,Distances.ALPHACARBON);
			
			Matrix M = gnm.getLambdaMatrix();
			
			int n = M.getRowDimension();
						
			// Correct values are calculated with the oGNM webserver
			assertEquals(0, M.get(n-1, n-1),0.00001);
			assertEquals(0.28314, M.get(n-2, n-2),0.00001);
			assertEquals(0.45745, M.get(n-3, n-3),0.00001);
			assertEquals(0.60259, M.get(n-4, n-4),0.00001);
			assertEquals(0.71863, M.get(n-5, n-5),0.00001);
			assertEquals(4.04574, M.get(n-20, n-20),0.00001);
			assertEquals(0, M.get(13, 96),0.00001);
			assertEquals(0, M.get(11, 14),0.00001);
			assertEquals(0, M.get(26, 49),0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetEigenvectorMatrix() {
	try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			Structure[] chains = s.getChains();
			Structure input = chains[0];
			input = input.getSubStructure(input.getAlphaCarbonIndexSet());	
			
			GNM gnm = new GNM(input,0.7,Distances.ALPHACARBON);
			
			Matrix M = gnm.getEigenvectorMatrix();
				
			int n = M.getRowDimension();
						
			// Correct values are calculated with the oGNM webserver
			assertEquals(-0.2999, M.get(0, n-2),0.0001);
			assertEquals(-0.1867, M.get(7, n-2),0.0001);
			assertEquals(0.1172, M.get(20, n-3),0.0001);
			assertEquals(-0.09673, M.get(28, n-3),0.0001);
			assertEquals(0.04106, M.get(1, n-7),0.0001);
			assertEquals(-0.03499, M.get(12, n-7),0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMSFProfile() {

		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			Structure[] chains = s.getChains();
			Structure input = chains[0];
			input = input.getSubStructure(input.getAlphaCarbonIndexSet());	
			
			GNM gnm = new GNM(input,0.7,Distances.ALPHACARBON);
			
			ArrayList<Double> msf = gnm.getMSFProfile();
			
			// Correct values are calculated with the oGNM webserver
			assertEquals(0.7053, msf.get(0),0.0001);
			assertEquals(0.3194, msf.get(7),0.0001);
			assertEquals(0.3247, msf.get(19),0.0001);
			assertEquals(0.1198, msf.get(63),0.0001);
			assertEquals(0.2570, msf.get(99),0.0001);
			assertEquals(0.7836, msf.get(102),0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	
		
		
	}

}
