/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import jgromacs.data.Atom;
import jgromacs.data.IndexSet;
import jgromacs.data.Point3D;
import jgromacs.data.PointList;
import jgromacs.data.Residue;
import jgromacs.data.Structure;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class ResidueTest extends TestCase {

	@Test
	public void testGetNumberOfAtoms() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");

			assertEquals(17,s.getResidueByIndex(3).getNumberOfAtoms());	
			assertEquals(16,s.getResidueByIndex(7).getNumberOfAtoms());
			assertEquals(14,s.getResidueByIndex(54).getNumberOfAtoms());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAtomByIndex() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			Residue r = s.getResidueByIndex(54);
			
			Atom a = r.getAtomByIndex(811);
			assertTrue(a.isBetaCarbon());
			
			a = r.getAtomByIndex(821);
			assertTrue(a.isCarbonylOxygen());	
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAtomByName() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			Residue r = s.getResidueByIndex(54);
			
			Atom a = r.getAtomByName("HD2");
			assertEquals(819,a.getIndex());
			
			a = r.getAtomByName("N");
			assertEquals(808,a.getIndex());
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGet1LetterCode() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(7);
			assertEquals("V",r.get1LetterCode());
			
			r = s.getResidueByIndex(8);
			assertEquals("R",r.get1LetterCode());
			
			r = s.getResidueByIndex(9);
			assertEquals("V",r.get1LetterCode());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGet3LetterCode() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(7);
			assertEquals("Val",r.get3LetterCode());
			
			r = s.getResidueByIndex(8);
			assertEquals("Arg",r.get3LetterCode());
			
			r = s.getResidueByIndex(9);
			assertEquals("Val",r.get3LetterCode());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetCombinedCode() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(7);
			assertEquals("7Val",r.getCombinedCode());
			
			r = s.getResidueByIndex(8);
			assertEquals("8Arg",r.getCombinedCode());
			
			r = s.getResidueByIndex(9);
			assertEquals("9Val",r.getCombinedCode());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testRemoveAtomByIndex() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(7);
			Atom a = r.getAtom(4);
			
			assertTrue(r.isAtomIn(a));
			
			r.removeAtomByIndex(a.getIndex()); // Removing atom
			
			assertTrue(!r.isAtomIn(a));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetNTerminalNitrogen() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(7);
			
			Atom a = r.getNTerminalNitrogen();
			
			assertEquals(109, a.getIndex());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAlphaCarbon() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(7); // Val
			
			Atom a = r.getAlphaCarbon();
			
			assertEquals(111, a.getIndex());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetBetaCarbon() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(7); // Val
			Atom a = r.getBetaCarbon();
			assertEquals(113, a.getIndex());
			
			r = s.getResidueByIndex(22); // Gly
			a = r.getBetaCarbon();
			assertEquals(null, a);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetGammaCarbon() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(28); // Asn
			Atom a = r.getGammaCarbon();
			assertEquals(412, a.getIndex());
			
			r = s.getResidueByIndex(49); // Ser
			a = r.getGammaCarbon();
			assertEquals(null, a);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}
	
	@Test
	public void testGetDeltaCarbon() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(41); // Arg
			Atom a = r.getDeltaCarbon();
			assertEquals(613, a.getIndex());
			
			r = s.getResidueByIndex(74); // Ala
			a = r.getDeltaCarbon();
			assertEquals(null, a);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}
	
	@Test
	public void testGetEpsilonCarbon() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(81); // Phe
			Atom a = r.getEpsilonCarbon();
			assertEquals(1232, a.getIndex());
			
			r = s.getResidueByIndex(74); // Ala
			a = r.getEpsilonCarbon();
			assertEquals(null, a);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}
	
	@Test
	public void testGetZetaCarbon() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg
			Atom a = r.getZetaCarbon();
			assertEquals(100, a.getIndex());
			
			r = s.getResidueByIndex(74); // Ala
			a = r.getZetaCarbon();
			assertEquals(null, a);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetCTerminalCarbon() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(41); // Arg
			Atom a = r.getCTerminalCarbon();
			assertEquals(625, a.getIndex());
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetCarbonylOxygen() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(41); // Arg
			Atom a = r.getCarbonylOxygen();
			assertEquals(626, a.getIndex());
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			for (int i = 85; i < 109; i++) control.addIndex(i);
			
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetHydrogenAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getHydrogenAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			control.addIndex(86);
			control.addIndex(88);
			control.addIndex(90);
			control.addIndex(91);
			control.addIndex(93);
			control.addIndex(94);
			control.addIndex(96);
			control.addIndex(97);
			control.addIndex(99);
			control.addIndex(102);
			control.addIndex(103);
			control.addIndex(105);
			control.addIndex(106);
					
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetHeavyAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getHeavyAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			control.addIndex(85);
			control.addIndex(87);
			control.addIndex(89);
			control.addIndex(92);
			control.addIndex(95);	
			control.addIndex(98);
			control.addIndex(100);
			control.addIndex(101);
			control.addIndex(104);
			control.addIndex(107);
			control.addIndex(108);
					
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetBackBoneAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getBackBoneAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			control.addIndex(85);
			control.addIndex(87);
			control.addIndex(107);
					
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMainChainAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getMainChainAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			control.addIndex(85);
			control.addIndex(87);
			control.addIndex(107);
			control.addIndex(108);
					
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMainChainPlusCbAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getMainChainPlusCbAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			control.addIndex(85);
			control.addIndex(87);
			control.addIndex(89);
			control.addIndex(107);
			control.addIndex(108);
					
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMainChainPlusHAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getMainChainPlusHAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			control.addIndex(85);
			control.addIndex(86);
			control.addIndex(87);
			control.addIndex(88);
			control.addIndex(107);
			control.addIndex(108);
			
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetSideChainAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getSideChainAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			control.addIndex(89);
			control.addIndex(90);
			control.addIndex(91);
			control.addIndex(92);
			control.addIndex(93);
			control.addIndex(94);
			control.addIndex(95);
			control.addIndex(96);
			control.addIndex(97);
			control.addIndex(98);
			control.addIndex(99);
			control.addIndex(100);
			control.addIndex(101);
			control.addIndex(102);
			control.addIndex(103);
			control.addIndex(104);
			control.addIndex(105);
			control.addIndex(106);
			
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetSideChainMinusHAtomIndices() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg

			IndexSet iset = r.getSideChainMinusHAtomIndices();
			iset.setName("");
			
			IndexSet control = new IndexSet("");
			control.addIndex(89);
			control.addIndex(92);
			control.addIndex(95);
			control.addIndex(98);
			control.addIndex(100);
			control.addIndex(101);
			control.addIndex(104);
			
			assertEquals(control, iset);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetAtomOfIndexCoordinates() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); // Arg
			
			r.setAtomOfIndexCoordinates(108, new Point3D(1,1,1));
			
			Atom a = r.getAtomByIndex(108);
			
			assertEquals(new Point3D(1,1,1), a.getCoordinates());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetAllAtomCoordinates() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			Residue r = s.getResidueByIndex(6); // Arg
			
			PointList pl = new PointList();
			for (int i = 0; i < 24; i++) pl.addPoint(new Point3D(1,1,1));
			
			r.setAllAtomCoordinates(pl);
			
			for (int i = 0; i < 24; i++) 
			assertEquals(new Point3D(1,1,1),r.getAtom(i).getCoordinates());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAllAtomCoordinates() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			Residue r = s.getResidueByIndex(6); // Arg
			
			PointList pl = new PointList();
			for (int i = 0; i < 24; i++) pl.addPoint(new Point3D(1,1,1));
			
			r.setAllAtomCoordinates(pl);
			
			PointList pl2 = r.getAllAtomCoordinates();
			
			assertEquals(pl,pl2);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAlphaCarbonCoordinates() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); 
			assertEquals(new Point3D(1.778,2.382,3.566),r.getAlphaCarbonCoordinates());
			
			r = s.getResidueByIndex(59); 
			assertEquals(new Point3D(1.918,3.200,2.987),r.getAlphaCarbonCoordinates());
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testDistanceAlphaCarbons() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); 
			Residue r2 = s.getResidueByIndex(59); 
			
			assertEquals(1.0119,r.distanceAlphaCarbons(r2),0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testDistanceClosest() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); 
			Residue r2 = s.getResidueByIndex(59); 
			
			ArrayList<Double> dists = new ArrayList<Double>();
			for (int i = 0; i < r.getNumberOfAtoms(); i++) {
				for (int j = 0; j < r2.getNumberOfAtoms(); j++) {
					dists.add(r.getAtom(i).distance(r2.getAtom(j)));
				}
			}

			assertEquals(Collections.min(dists),r.distanceClosest(r2),0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testDistanceClosestHeavy() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Residue r = s.getResidueByIndex(6); 
			Residue r2 = s.getResidueByIndex(59); 
			
			ArrayList<Double> dists = new ArrayList<Double>();
			for (int i = 0; i < r.getNumberOfAtoms(); i++) {
				for (int j = 0; j < r2.getNumberOfAtoms(); j++) {
					if ((!r.getAtom(i).isHydrogenAtom())&&(!r2.getAtom(j).isHydrogenAtom()))
					dists.add(r.getAtom(i).distance(r2.getAtom(j)));
				}
			}

			assertEquals(Collections.min(dists),r.distanceClosestHeavy(r2),0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsAminoAcid() {
		
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			
			Residue r = s.getResidueByIndex(1377); 				
			assertTrue(r.isAminoAcid());
			
			r = s.getResidueByIndex(1315); 				
			assertTrue(r.isAminoAcid());
			
			r = s.getResidueByIndex(163); 				
			assertTrue(!r.isAminoAcid());	
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsWater() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			
			Residue r = s.getResidueByIndex(30); 				
			assertTrue(r.isWater());
			
			r = s.getResidueByIndex(163); 				
			assertTrue(r.isWater());
			
			r = s.getResidueByIndex(1377); 				
			assertTrue(!r.isWater());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsOther() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example2.pdb");
			
			Residue r = s.getResidueByIndex(150); 				
			assertTrue(r.isOther());
			
			r = s.getResidueByIndex(153); 				
			assertTrue(r.isOther());
			
			r = s.getResidueByIndex(146); 				
			assertTrue(!r.isOther());
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsAtomIn() {

		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			
			Residue r = s.getResidueByIndex(1377); 				
			Atom a = r.getAtom(0);
			
			assertTrue(r.isAtomIn(a));
			
			r.removeAtom(a);
			
			assertTrue(!r.isAtomIn(a));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

}
